%% plasma parameters
n0 = 6.5e19; % electron density, m^-3
R = 1.85; % major radius, m
a = 0.45; % minor radius, m
w = 37e6*2*pi; % angular frequency, rad/s
B0 = 2.5; % magnetic field, T
n_para = 5; % parallel refraction index
k_para = n_para*w/const.c;

n_bound = 3; % sum limits for In

%% ne data
edgefile = fopen('edge.txt','r');
edgedata = fscanf(edgefile, '%f %f', [2, inf]);
fclose(edgefile);

% x = edgedata(1,:) reverse
x_edge = fliplr(edgedata(1,:)); % unit m
ne_edge = fliplr(edgedata(2,:))*1e19; % unit m^-3

% interpolation: ne_edge
x_edge_inter = linspace(min(x_edge), max(x_edge), 100);
ne_edge_inter = interp1(x_edge, ne_edge, x_edge_inter, 'spline');
x = linspace(1.6, min(x_edge), 100);
ne = sqrt(1-((x-R)/a).^2)*n0;

% concatenate x and x_edge
x = [x, x_edge_inter];
ne = [ne, ne_edge_inter];

% Plasma profiles
nH = ne .* 0.1;
nD = ne .* 0.9;
mH = const.mp;
mD = 2*const.mp;
Bx = B0 * (R./x);

create_H = @(nH) species('H', mH, const.qe, nH, 1e8, 1e8, 0);
create_D = @(nD) species('D', mD, const.qe, nD, 1e8, 1e8, 0);
create_e = @(ne) species('e', const.me, -const.qe, ne, 1e8, 1e8, 0);

H_list = arrayfun(create_H,nH);
D_list = arrayfun(create_D,nD);
e_list = arrayfun(create_e,ne);

kx = zeros(size(x));

%% solving
start_time = cputime;
for i = 1:size(kx,2)
    if (i == 1) 
        kx0 = 31.5157 + 0.0676i;
    else
        kx0 = kx(i-1);
    end
    wav = wave(k_para, kx0, w);
    kx(i) = solver([H_list(i),D_list(i),e_list(i)],...
                    wav, Bx(i), n_bound);
    disp(kx(i))
end
runtime = cputime-start_time;
save kxfile.mat kx

%% plot
re_kx = real(kx);
im_kx = imag(kx);
% plot re_kx and im_kx in the same figure
figure
plot(x, re_kx, 'r', x, im_kx, 'b')
xlabel('x (m)')
ylabel('kx (1/m)')
legend('real(kx)', 'imag(kx)')
title('kx vs x')

