classdef tensors
    properties
        resp % responses from all species in the plasma
        w
        k_para
        k_perp
        kappa
    end

    methods
        function obj = tensors(response_list, wav)
            arguments
                response_list (1, :) response
                wav wave
            end
            obj.resp = response_list;
            obj.w = wav.w;
            obj.k_para = wav.k_para;
            obj.k_perp = wav.k_perp;
        end

        function K = KTensor(all)

            [K0_elem, K1_elem, K2_elem,...
             K3_elem, K4_elem, K5_elem] = arrayfun(@K_elems, all.resp);
            
            K0 = 2 * sum(K0_elem);
            K1 = 1 + sum(K1_elem);
            K2 = 1i * sum(K2_elem);
            K3 = 1 - sum(K3_elem);
            K4 = sum(K4_elem);
            K5 = 1i * sum(K5_elem);

            K = [K1,  K2,    K4;...
                 -K2, K1+K0, -K5;...
                 K4,  K5,    K3];
        end

        function M = MTensor(all)
            k = (all.w^2/const.c^2) .* all.KTensor();
            kz = all.k_para;
            kx = all.k_perp;
            M = [k(1,1) - kz^2, k(1,2),               k(1,3) + kx*kz;...
                k(2,1),         k(2,2) - kz^2 - kx^2, k(2,3);...
                k(3,1) + kz*kx, k(3,2),               k(3,3) - kx^2];
        end

        function detM = detM(all)
            detM = det(all.MTensor());
        end
    end
end